package upv.pauchorroyanguas.com.encryptorpcy;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.os.AsyncTask;
import android.preference.PreferenceManager;
import android.support.v7.app.ActionBarActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import org.w3c.dom.Text;

import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;

import upv.pauchorroyanguas.com.encryptorpcy.Utils.PasswordHash;

/**
 *  Author: Pau Chorro
 *  Date: 10/11/15
 *
 *  CreatePassword Activity: Users should insert their password for the app
 */
public class CreatePasswordActivity extends ActionBarActivity {

    Button btnCreatePass;
    TextView tvPassword;
    TextView tvRepeteadPassword;

    String password;
    String repeteadPassword;

    private static final String LOG_TAG = "TAG_CRE_PASS_ACT";
    private final Activity activity = this;

    ProgressDialog pd;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        //Configure theme
        SharedPreferences pref = PreferenceManager
                .getDefaultSharedPreferences(this);
        String themeName = pref.getString("themeApp", "1");

        if(themeName.equalsIgnoreCase("0")) {

            setTheme(R.style.ThemeEncryptorPCY);
        }
        else if(themeName.equalsIgnoreCase("1"))
        {
            setTheme(R.style.ThemeEncryptorPCYLightBlue);
        }
        else
        {
            setTheme(R.style.ThemeEncryptorPCYYellow);
        }

        setContentView(R.layout.activity_create_password);

        btnCreatePass = (Button) findViewById(R.id.btnCreatePass);
        tvPassword = (EditText) findViewById(R.id.tvPassword);
        tvRepeteadPassword = (EditText) findViewById(R.id.tvPassword2);

        btnCreatePass.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                password = tvPassword.getText().toString();
                repeteadPassword = tvRepeteadPassword.getText().toString();

                Log.e(LOG_TAG, password);
                Log.e(LOG_TAG, repeteadPassword);


                if (password.equals(repeteadPassword) && !password.equals("")) {
                    if(password.length() < 4)
                    {
                        Toast.makeText(getApplicationContext(),
                                getStringResources(R.string.ToastErrorLength), Toast.LENGTH_LONG).show();
                    }
                    else {
                        SendTask task = new SendTask(getApplicationContext());
                        task.execute();
                    }
                } else {
                    Toast.makeText(getApplicationContext(),
                            getStringResources(R.string.ToastErrorSameName), Toast.LENGTH_LONG).show();
                }
            }
        });
    }

    /**
     * It will dismiss the dialog for avoid the leak
     */
    @Override
    public void onPause(){

        super.onPause();
        if(pd != null)
            pd.dismiss();
    }

   /* @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_create_password, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }*/

    /**
     * Launch Login Activity
     * @param view Prepared for method OnClick
     */
    public void launchLogin(View view)
    {
        Intent i = new Intent(this, LoginActivity.class);
        startActivity(i);
    }

    /*
    No networking allowed on the main thread.
    */
    private class SendTask extends AsyncTask<String,String,String>{

        int result;
        boolean failure = false;
        Context context;

        public SendTask(Context context)
        {
            this.context = context;
        }

        @Override
        protected void onPreExecute(){
            pd = ProgressDialog.show(CreatePasswordActivity.this,"",getStringResources(R.string.CreatingPassword), true,false);
        }

        @Override
        protected String doInBackground(String...strings){

            //do networking here
            //result = account.prepare();
            SharedPreferences prefs =
                    getSharedPreferences("Password", Context.MODE_PRIVATE);

            try {

                String hash = PasswordHash.createHash(password,
                        context);
                String secondHash = PasswordHash.createHash(password,
                        context);
                if(hash.equals(secondHash)) {
                    Log.e(LOG_TAG, "FAILURE: TWO HASHES ARE EQUAL!");
                    failure = false;
                }
                String wrongPassword = password +1;
                if(PasswordHash.validatePassword(wrongPassword, hash)) {
                    Log.e(LOG_TAG, "FAILURE: WRONG PASSWORD ACCEPTED!");
                    failure = true;
                }
                if(!PasswordHash.validatePassword(password, hash)) {
                    Log.e(LOG_TAG, "FAILURE: GOOD PASSWORD NOT ACCEPTED!");
                    failure = true;
                }

                if(failure) {

                    Log.e(LOG_TAG, "TESTS FAILED!");

                }
                else {

                    Log.e(LOG_TAG, "TESTS PASSED!");

                    SharedPreferences.Editor editor = prefs.edit();
                    editor.putString("pass", secondHash);
                    editor.commit();

                    Log.e(LOG_TAG,"Has1: "  + hash);
                    Log.e(LOG_TAG,"Has2: "  + secondHash);

                }


            } catch (NoSuchAlgorithmException e) {
                e.printStackTrace();
            } catch (InvalidKeySpecException e) {
                e.printStackTrace();
            }
            return null;
        }

        @Override
        protected void onPostExecute(String unused){

            //check result
            pd.dismiss();


            if(!failure)
            {
                Toast.makeText(context,
                        getStringResources(R.string.GOODPasswordCreated), Toast.LENGTH_LONG);

                launchLogin(null);
            }
            else
            {
                Toast.makeText(context,getStringResources(R.string.ErrorPassword) +
                        "\n" + getStringResources(R.string.TryAgain) , Toast.LENGTH_LONG);
            }

        }

    }

    /**
     * Onback go to Home
     */
    @Override
    public void onBackPressed() {

        Intent setIntent = new Intent(Intent.ACTION_MAIN);
        setIntent.addCategory(Intent.CATEGORY_HOME);
        setIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        startActivity(setIntent);
    }

    /**
     * It will get the string from the resources Values
     * @param id Id of the resource String
     * @return String String from the id resource
     */
    public String getStringResources(int id)
    {
        Resources res = getResources();
        return res.getString(id);
    }

}
