package upv.pauchorroyanguas.com.encryptorpcy;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.os.AsyncTask;
import android.preference.PreferenceManager;
import android.support.v7.app.ActionBarActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import java.security.NoSuchAlgorithmException;
import java.security.spec.InvalidKeySpecException;

import upv.pauchorroyanguas.com.encryptorpcy.Utils.FolderUtil;
import upv.pauchorroyanguas.com.encryptorpcy.Utils.PasswordHash;

/**
 *  Author: Pau Chorro
 *  Date: 01/11/15
 *
 *  Login Activity: Users should insert their password to decrypt the content
 */
public class LoginActivity extends ActionBarActivity {

    ProgressDialog pd;

    Button logIn;
    TextView tvPassword;
    TextView tvResult;
    //Constants Folders
    private static String mainFolder = "EncryptorPCY";

    private static final String LOG_TAG = "TAG_LOGIN_ACT";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        //Configure theme
        SharedPreferences pref = PreferenceManager
                .getDefaultSharedPreferences(this);
        String themeName = pref.getString("themeApp", "1");

        if(themeName.equalsIgnoreCase("0")) {

            setTheme(R.style.ThemeEncryptorPCY);
        }
        else if(themeName.equalsIgnoreCase("1"))
        {
            setTheme(R.style.ThemeEncryptorPCYLightBlue);
        }
        else
        {
            setTheme(R.style.ThemeEncryptorPCYYellow);
        }

        setContentView(R.layout.activity_login);

        //Create Folders
        FolderUtil.createHiddenFolder(mainFolder);

        //Views
        logIn = (Button) findViewById(R.id.btnLogin);
        tvPassword = (TextView) findViewById(R.id.tvPassword);
        tvResult = (TextView) findViewById(R.id.tvResult);

        //View actions
        logIn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                //Launch main activity
                //launchMain(null);
                //launchMainTest(null);

                if ("".equalsIgnoreCase(tvPassword.getText().toString())) {
                    Toast.makeText(getApplicationContext(), getStringResources(R.string.PassworCannotBeEmpty), Toast.LENGTH_LONG).show();
                } else {
                    SendTask task = new SendTask(getApplicationContext(),
                            tvPassword.getText().toString());
                    task.execute();
                }
            }
        });

    }

    @Override
    public void onPause(){

        super.onPause();

        if(pd != null)
            pd.dismiss();

    }

    /*@Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_login, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;

        }

        return super.onOptionsItemSelected(item);
    }*/

    /**
     * Launch Main Activity
     * @param view Prepared for method OnClick
     */
    public void launchMain(View view)
    {
        Intent i = new Intent(this, MainActivity.class);
        startActivity(i);
    }


    /**
     * Onback go to Home
     */
    @Override
    public void onBackPressed() {

        Intent setIntent = new Intent(Intent.ACTION_MAIN);
        setIntent.addCategory(Intent.CATEGORY_HOME);
        setIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        startActivity(setIntent);
    }

    /**
     *  No networking allowed on the main thread.
     */
    private class SendTask extends AsyncTask<String,String,String> {

        int result;
        boolean failure = false;
        Context context;
        SharedPreferences prefs;
        String hash;
        String password;

        public SendTask(Context context, String password)
        {
            this.context = context;
            this.password = password;
        }

        @Override
        protected void onPreExecute(){
            pd = ProgressDialog.show(LoginActivity.this, "", getStringResources(R.string.CheckPassword), true, false);
            prefs = getSharedPreferences("Password", Context.MODE_PRIVATE);
        }

        @Override
        protected String doInBackground(String...strings){

            //do networking here
            //result = account.prepare();

            try {

                // Test password validation
                System.out.println("Running tests...");

                //String password = tvPassword.getText().toString();
                hash = PasswordHash.createHash(password, getApplicationContext());
                String secondHash = PasswordHash.createHash(password, getApplicationContext());
                if(hash.equals(secondHash)) {
                    Log.e(LOG_TAG, "FAILURE: TWO HASHES ARE EQUAL!");
                    failure = false;
                }
                String wrongPassword = password +1;
                if(PasswordHash.validatePassword(wrongPassword, hash)) {
                    Log.e(LOG_TAG, "FAILURE: WRONG PASSWORD ACCEPTED!");
                    failure = true;
                }
                if(!PasswordHash.validatePassword(password, hash)) {
                    Log.e(LOG_TAG, "FAILURE: GOOD PASSWORD NOT ACCEPTED!");
                    failure = true;
                }


            } catch (NoSuchAlgorithmException e) {
                e.printStackTrace();
            } catch (InvalidKeySpecException e) {
                e.printStackTrace();
            }
            return null;
        }

        @Override
        protected void onPostExecute(String unused){

            //check result
            pd.dismiss();

            if(failure) {
                Log.e(LOG_TAG, "TESTS FAILED!");
                tvResult.setVisibility(View.VISIBLE);
                Toast.makeText(context, getStringResources(R.string.IncorrectPassword), Toast.LENGTH_LONG).show();
            }
            else {
                Log.e(LOG_TAG, "TESTS PASSED!");

                String pass = prefs.getString("pass", "?");

                Log.e(LOG_TAG, pass + ", " + hash);

                if(pass.equals(hash))
                {
                    tvResult.setVisibility(View.GONE);
                    launchMain(null);
                }
                else
                {
                    tvResult.setVisibility(View.VISIBLE);
                    Toast.makeText(context, getStringResources(R.string.IncorrectPassword), Toast.LENGTH_LONG).show();
                }
            }

        }
    }

    /**
     * It will get the string from the resources Values
     * @param id Id of the resource String
     * @return String String from the id resource
     */
    public String getStringResources(int id)
    {
        Resources res = getResources();
        return res.getString(id);
    }


}
