package upv.pauchorroyanguas.com.encryptorpcy.Utils;

import android.os.Environment;
import android.util.Log;

import org.apache.commons.io.IOUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.CipherOutputStream;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.SecretKeySpec;

import upv.pauchorroyanguas.com.encryptorpcy.Exception.CryptoException;

/**
 * Author: Pau Chorro
 * Date: 10/11/15
 *
 * CryptoUtils Class
 */
public class CryptoUtils {
    private static final String ALGORITHM = "AES";
    private static final String TRANSFORMATION = "AES";

    private static String mainPath = Environment.getExternalStorageDirectory() +
            "/";

    /**
     * Call method doCrypto in Encrypt Mode
     * @param key String key to encrypt
     * @param inputFile File file which will be encrypted or decrypted
     * @param outputFile File
     * @throws CryptoException
     */
    public static void encrypt(String key, File inputFile, File outputFile)
            throws CryptoException {
        doCrypto(Cipher.ENCRYPT_MODE, key, inputFile, outputFile);
    }

    public static void decrypt(String key, File inputFile, File outputFile)
            throws CryptoException {
        doCrypto(Cipher.DECRYPT_MODE, key, inputFile, outputFile);
    }

    private static void doCrypto(int cipherMode, String key, File inputFile,
                                 File outputFile) throws CryptoException {
        try {
            Key secretKey = new SecretKeySpec(key.getBytes(), ALGORITHM);
            Cipher cipher = Cipher.getInstance(TRANSFORMATION);
            cipher.init(cipherMode, secretKey);

            FileInputStream inputStream = new FileInputStream(inputFile);
            FileOutputStream outputStream = new FileOutputStream(outputFile);


            CipherOutputStream cos = new CipherOutputStream(outputStream, cipher);

            //Now read from input and write to output using your favorite utilities library
            //Guava and Apache Commons IO are good examples.
            IOUtils.copy(inputStream, cos);
            //Remember to close streams if the previous call didn't (preferably in a finally block)


            //No funciona
            /*byte[] inputBytes = new byte[1];
            int bytesRead;
            byte[] output;

            while((bytesRead = inputStream.read(inputBytes, 0, 1)) != -1)
            {
                output = cipher.update(inputBytes, 0 , bytesRead);

                if(output != null)
                    outputStream.write(output);
            }*/

            inputStream.close();
            outputStream.close();

        } catch (NoSuchPaddingException | NoSuchAlgorithmException
                | InvalidKeyException
                | IOException ex) {
            throw new CryptoException("Error encrypting/decrypting file", ex);
        }
    }

    /**
     * Temporal crypto for images....
     * @param cipherMode
     * @param key
     * @param inputFile
     * @param outputFile
     * @throws CryptoException
     */
    private static void doCryptoImage(int cipherMode, String key, File inputFile,
                                 File outputFile) throws CryptoException {
        try {
            Key secretKey = new SecretKeySpec(key.getBytes(), ALGORITHM);
            Cipher cipher = Cipher.getInstance(TRANSFORMATION);
            cipher.init(cipherMode, secretKey);


            FileInputStream inputStream = new FileInputStream(inputFile);
            FileOutputStream outputStream = new FileOutputStream(outputFile);

            byte[] inputBytes = new byte[(int) inputFile.length()];
            inputStream.read(inputBytes);

            byte[] outputBytes = cipher.doFinal(inputBytes);

            outputStream.write(outputBytes);

            inputStream.close();
            outputStream.close();

        } catch (NoSuchPaddingException | NoSuchAlgorithmException
                | InvalidKeyException
                | IOException ex) {
            throw new CryptoException("Error encrypting/decrypting file", ex);
        } catch (BadPaddingException e) {
            e.printStackTrace();
        } catch (IllegalBlockSizeException e) {
            e.printStackTrace();
        }
    }

    public static void encryptImage(String key, File inputFile, File outputFile)
            throws CryptoException {
        doCryptoImage(Cipher.ENCRYPT_MODE, key, inputFile, outputFile);
    }

    public static void decryptImage(String key, File inputFile, File outputFile)
            throws CryptoException {
        doCryptoImage(Cipher.DECRYPT_MODE, key, inputFile, outputFile);
    }

}
