package upv.pauchorroyanguas.com.encryptorpcy.Utils;

import android.content.ContentUris;
import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.provider.DocumentsContract;
import android.provider.MediaStore;
import android.util.Log;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

/**
 * Author: Pau Chorro
 * Date: 01/11/15
 *
 * FileUtil Class
 */
public class FileUtil {

    //Const
    private static final String LOG_TAG = "TAG_FILEUTIL";
    private static String mainPath = Environment.getExternalStorageDirectory() +
            "/";

    /**
     * Method to copy a file to another path
     * @param sourcePath Path of the original file to copy
     * @param relativeDestPath Path to copy the original file
     * @return Boolean True if the operation is okey, and false if it is not okey
     */
    public static boolean copyFile(String sourcePath, String relativeDestPath)
    {
        File source = new File(mainPath + sourcePath);
        File dest = new File(mainPath + relativeDestPath);
        try {
            FileUtils.copyFile(source, dest);

            return true;
        } catch (IOException e) {
            Log.e(LOG_TAG, " - copyFile() : " + e.getMessage(), e);
            return false;
        }
    }

    /**
     * Method to move a file to another path
     * @param sourcePath Path of the original file to move
     * @param relativeDestPath Path to move the original file
     * @return Boolean True if the operation is okey, and false if it is not okey
     */
    public static boolean moveFile(String sourcePath, String relativeDestPath)
    {
        String sourcePathAux = sourcePath.replace("file:/", "");
        File source = new File(sourcePathAux);
        File dest = new File(mainPath + relativeDestPath);
        try {
            FileUtils.moveFile(source, dest);

            return true;
        } catch (IOException e) {
            Log.e(LOG_TAG, " - moveFile() : " + e.getMessage(), e);
            return false;
        }
    }


    /**
     * Method to delete a file
     * @param sourcePath Path of the original file to delete
     * @return Boolean True if the operation is okey, and false if it is not okey
     */
    public static boolean deleteFile(String sourcePath)
    {
        File source = new File(sourcePath);

        try {
            FileUtils.forceDelete(source);

            return true;
        } catch (IOException e) {
            Log.e(LOG_TAG, " - deleteFile() : " + e.getMessage(), e);
            return false;
        }

    }

    /**
     * Method to get the bytes of a file, then it will be encrypted
     * @param sourcePath Path of the file
     * @return Byte[] Return the Bytes of the file in array of byte.
     */
    public static byte[] getBytesFile(String sourcePath)
    {
        File source = new File(sourcePath);

        try {

            return FileUtils.readFileToByteArray(source);

        } catch (IOException e) {
            Log.e(LOG_TAG, " - getBytesFile() : " + e.getMessage(), e);
            //TODO Change the return (now the catch return null)
            return null;
        }

    }

    /**
     *  It will return all files and directory
     * @param path Path of the folder
     * @return List<File> It will return the list files and folders...
     */
    public static List<File> getFilesAndDirectories(String path)
    {

        Log.e(LOG_TAG, " - getFilesAndDirectories() : " + mainPath+path);

        File f = new File(mainPath + path);
        File file[] = f.listFiles();

        List<File> files = new ArrayList<File>();

        if(file != null) {

            Log.e(LOG_TAG, "Files, " + "Size: " + file.length);

            for (int i = 0; i < file.length; i++) {
                files.add(file[i]);
                Log.e("Files", "FileName:" + file[i].getName());
            }

        }

        return files;
    }


    /**
     * It will get the name of the files and directories of the current folder
     * @param folder Is the path of the current folder (Example: path/currentfolder)
     * @return Vector<String> Name list of Files and directories
     */
    public static Vector<String> getNameFilesAndDirectories(String folder)
    {
        Vector<String> nameFiles = new Vector<String>();

        List<File> files = getFilesAndDirectories(folder);

        //Header
        nameFiles.add(folder);

        for(int i = 0; i < files.size(); i++)
        {
            nameFiles.add(files.get(i).getName());
        }

        //Foot
        //nameFiles.add("...");


        return nameFiles;
    }

    /**
     * It will return the extension of a file
     * @param file String of the name of the file (not path, just file or folder)
     * @return String Return the extension
     */
    public static String getExtension(String file)
    {
        Log.e(LOG_TAG, "getExtension: " + FilenameUtils.getExtension(file));
        return FilenameUtils.getExtension(file);
    }

    /**
     * Return a boolean that if it is true the file exists, but it is false it doesn't exist
     * @param pathFile Path of the file
     * @return True if the file exists, false if it doesn't exists
     */
    public static boolean fileExists(String pathFile)
    {
        File f = new File(mainPath + pathFile);
        return f.exists();
    }

    /**
     *
     * @param oldPath
     * @param newPath
     */
    public static void renameFile(String oldPath, String newPath)
    {
        File oldF = new File(mainPath + oldPath);
        File newF = new File(mainPath + newPath);

        oldF.renameTo(newF);

    }

    /**
     * Checks if external storage is available for read and write
     * @return boolean true or false
     */
    public static boolean isExternalStorageWritable() {
        String state = Environment.getExternalStorageState();
        if (Environment.MEDIA_MOUNTED.equals(state)) {
            return true;
        }
        return false;
    }

    /**
     * Checks if external storage is available to at least read
     * @return
     */
    public static boolean isExternalStorageReadable() {
        String state = Environment.getExternalStorageState();
        if (Environment.MEDIA_MOUNTED.equals(state) ||
                Environment.MEDIA_MOUNTED_READ_ONLY.equals(state)) {
            return true;
        }
        return false;
    }



    //------Private

    static File createExternalStoragePrivateFile(Context context, String nameFile) {

        // Create a path where we will place our private file on external
        // storage.
        File file = new File(context.getExternalFilesDir(null), nameFile);

        return file;

    }

    static void deleteExternalStoragePrivateFile(Context context, String nameFile) {
        // Get path for the file on external storage.  If external
        // storage is not currently mounted this will fail.
        File file = new File(context.getExternalFilesDir(null), nameFile);
        if (file != null) {
            file.delete();
        }
    }

    static boolean hasExternalStoragePrivateFile(Context context, String nameFile) {
        // Get path for the file on external storage.  If external
        // storage is not currently mounted this will fail.
        File file = new File(context.getExternalFilesDir(null), nameFile);
        if (file != null) {
            return file.exists();
        }
        return false;
    }


    /*
    GET URI
     */

    /**
     * Get a file path from a Uri. This will get the the path for Storage Access
     * Framework Documents, as well as the _data field for the MediaStore and
     * other file-based ContentProviders.
     *
     * @param context The context.
     * @param uri The Uri to query.
     * @author paulburke
     */
    public static String getPath(Context context, Uri uri) {

        final boolean isKitKat = Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT;

        // DocumentProvider
        if (isKitKat && DocumentsContract.isDocumentUri(context, uri)) {
            // ExternalStorageProvider
            if (isExternalStorageDocument(uri)) {
                final String docId = DocumentsContract.getDocumentId(uri);
                final String[] split = docId.split(":");
                final String type = split[0];

                if ("primary".equalsIgnoreCase(type)) {
                    return Environment.getExternalStorageDirectory() + "/" + split[1];
                }

                // TODO handle non-primary volumes
            }
            // DownloadsProvider
            else if (isDownloadsDocument(uri)) {

                final String id = DocumentsContract.getDocumentId(uri);
                final Uri contentUri = ContentUris.withAppendedId(
                        Uri.parse("content://downloads/public_downloads"), Long.valueOf(id));

                return getDataColumn(context, contentUri, null, null);
            }
            // MediaProvider
            else if (isMediaDocument(uri)) {
                final String docId = DocumentsContract.getDocumentId(uri);
                final String[] split = docId.split(":");
                final String type = split[0];

                Uri contentUri = null;
                if ("image".equals(type)) {
                    contentUri = MediaStore.Images.Media.EXTERNAL_CONTENT_URI;
                } else if ("video".equals(type)) {
                    contentUri = MediaStore.Video.Media.EXTERNAL_CONTENT_URI;
                } else if ("audio".equals(type)) {
                    contentUri = MediaStore.Audio.Media.EXTERNAL_CONTENT_URI;
                }

                final String selection = "_id=?";
                final String[] selectionArgs = new String[] {
                        split[1]
                };

                return getDataColumn(context, contentUri, selection, selectionArgs);
            }
        }
        // MediaStore (and general)
        else if ("content".equalsIgnoreCase(uri.getScheme())) {
            return getDataColumn(context, uri, null, null);
        }
        // File
        else if ("file".equalsIgnoreCase(uri.getScheme())) {
            return uri.getPath();
        }

        return null;
    }

    /**
     * Get the value of the data column for this Uri. This is useful for
     * MediaStore Uris, and other file-based ContentProviders.
     *
     * @param context The context.
     * @param uri The Uri to query.
     * @param selection (Optional) Filter used in the query.
     * @param selectionArgs (Optional) Selection arguments used in the query.
     * @return The value of the _data column, which is typically a file path.
     */
    public static String getDataColumn(Context context, Uri uri, String selection,
                                       String[] selectionArgs) {

        Cursor cursor = null;
        final String column = "_data";
        final String[] projection = {
                column
        };

        try {
            cursor = context.getContentResolver().query(uri, projection, selection, selectionArgs,
                    null);
            if (cursor != null && cursor.moveToFirst()) {
                final int column_index = cursor.getColumnIndexOrThrow(column);
                return cursor.getString(column_index);
            }
        } finally {
            if (cursor != null)
                cursor.close();
        }
        return null;
    }


    /**
     * @param uri The Uri to check.
     * @return Whether the Uri authority is ExternalStorageProvider.
     */
    public static boolean isExternalStorageDocument(Uri uri) {
        return "com.android.externalstorage.documents".equals(uri.getAuthority());
    }

    /**
     * @param uri The Uri to check.
     * @return Whether the Uri authority is DownloadsProvider.
     */
    public static boolean isDownloadsDocument(Uri uri) {
        return "com.android.providers.downloads.documents".equals(uri.getAuthority());
    }

    /**
     * @param uri The Uri to check.
     * @return Whether the Uri authority is MediaProvider.
     */
    public static boolean isMediaDocument(Uri uri) {
        return "com.android.providers.media.documents".equals(uri.getAuthority());
    }

}
