package upv.pauchorroyanguas.com.encryptorpcy.Utils;

import android.os.Environment;
import android.util.Log;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.filefilter.TrueFileFilter;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

/**
 * Author: Pau Chorro
 * Date: 01/11/15
 *
 * FolderUtil Class
 */
public class FolderUtil {

    //Const
    private static final String LOG_TAG = "TAG_FODERUTIL";
    private static String mainPath = Environment.getExternalStorageDirectory() + "/";

    /**
     * Crate a hidden folder with a . before its in the main path (Android/data/)
     * @param nameFolder Name of the folder which will be created
     * @return Boolean True if the operation is okey, and false if it is not okey
     */
    public static boolean createHiddenFolder(String nameFolder)
    {
        try {

            File ruta = new File(mainPath + "." +nameFolder);
            if (!ruta.exists()) {
                ruta.mkdirs();

                final File fi = new File(ruta, "readme.txt");
                FileOutputStream f = new FileOutputStream(fi, true);
                String texto = "Please don't delete this Folder in other File Explorer App\n";
                f.write(texto.getBytes());
                texto = "Por favor no borre esta carpeta desde otra Aplicación de Explorador de Archivos";
                f.write(texto.getBytes());
                f.close();
                return true;
            }
        } catch (Exception e) {
            Log.e(LOG_TAG," - createHiddenFolder() : " +  e.getMessage(), e);
            return false;
        }
        return false;
    }

    /**
     * Create a folder in the path indicate from main path (Android/data/)
     * @param relativePath Relative path of the folder where it will be created (It is not needed "/")
     * @param nameFolder Name of the folder which will be created (It is not needed "/")
     * @return Boolean True if the operation is okey, and false if it is not okey
     */
    public static boolean createFolder(String relativePath, String nameFolder)
    {
        try {

            File ruta = new File(mainPath + relativePath  + "/" +nameFolder);
            if (!ruta.exists()) {
                Log.e(LOG_TAG," - createFolder() : folder created --> " +ruta.getAbsolutePath()  );
                ruta.mkdirs();

                /*final File fi = new File(ruta, "readme.txt");
                FileOutputStream f = new FileOutputStream(fi, true);
                String texto = "Please Readme (blablablalblalba)";
                f.write(texto.getBytes());
                f.close();*/
                return true;
            }
        } catch (Exception e) {
            Log.e(LOG_TAG," - createFolder() : " +  e.getMessage(), e);
            return false;
        }
        return false;
    }

    /**
     * Copy a folder to other path
     * @param sourcePath Path of the original folder
     * @param destPath Path of the destination of the copy of the original folder
     * @return Boolean True if the operation is okey, and false if it is not okey
     */
    public static boolean copyFolder(String sourcePath, String destPath)
    {
        File source = new File(mainPath + sourcePath);
        File dest = new File(mainPath + destPath);

        try {
            FileUtils.copyDirectory(source, dest);
            return true;
        } catch (IOException e) {
            Log.e(LOG_TAG, " - copyFolder() : " + e.getMessage(), e);
            return false;

        }

    }

    /**
     * Clean the folder without deleting it
     * @param pathDirectory Path of the directory to clean
     * @return Boolean True if the operation is okey, and false if it is not okey
     */
    public static boolean cleanFolder(String pathDirectory)
    {
        File directory = new File(pathDirectory);

        try {
            FileUtils.cleanDirectory(directory);
            return true;
        } catch (IOException e) {
            Log.e(LOG_TAG, " - cleanFolder() : " + e.getMessage(), e);
            return false;
        }

    }

    /**
     * Depcrecated: It is better to use the FileUtil method:  deleteFile()
     * Delete the directory recursevily
     * @param pathDirectory Path of the directory to delete
     * @return  Boolean True if the operation is okey, and false if it is not okey
     */
    @Deprecated
    public static boolean deleteFolder(String pathDirectory)
    {
        File directory = new File(pathDirectory);

        try {
            FileUtils.deleteDirectory(directory);
            return true;
        } catch (IOException e) {
            Log.e(LOG_TAG, " - deleteFolder() : " + e.getMessage(), e);
            return false;
        }

    }

    /**
     * Move the directory to another path
     * @param sourcePath Original path of the directory to move
     * @param destPath Destination path of the directory
     * @return Boolean True if the operation is okey, and false if it is not okey
     */
    public static boolean moveFolder(String sourcePath, String destPath)
    {
        File source = new File(mainPath + sourcePath);
        File dest = new File(mainPath + destPath);

        try {
            FileUtils.moveDirectory(source, dest);
            return true;
        } catch (IOException e) {
            Log.e(LOG_TAG, " - moveFolder() : " + e.getMessage(), e);
            return false;
        }
    }

    /**
     * It will get all the folders inside the other folder
     * @param path Path of the folder
     * @return List<File> It will return the list of folder not files...
     */
    public static List<File>  getListFolders(String path)
    {
        File dir = new File(mainPath + path);
        List<File> files =
                (List<File>) FileUtils.listFiles(dir, TrueFileFilter.INSTANCE, TrueFileFilter.INSTANCE);

        List<File> folders = new ArrayList<File>();

        for(File file : files)
        {
            if(file.isDirectory())
                folders.add(file);
        }

        return folders;
    }

    /**
     * Get a Vector of String with the folders of the Main folder
     * @return Vector<String> Name of the folders inn the Main Folder
     */
    public static Vector<String> getNameFolders(String folder)
    {
        Vector<String> nameFolders = new Vector<String>();

        List<File> files = FolderUtil.getListFolders(folder);

        //Header
        nameFolders.add(folder);

        for(int i = 0; i < files.size(); i++)
        {
            nameFolders.add(files.get(i).getName());
        }

        //Foot
        nameFolders.add("...");


        return nameFolders;
    }

    /**
     * It will check if the path is a folder or not, in case of be a folder, it will return true,
     * in case of be a file and not a directory it will return false
     * @param path Path of the directory (Example android/data/encryptor/upv)
     * @return boolean True if it is a directory and false if it is not a directory
     */
    public static boolean isFolder(String path)
    {
        File dir = new File(mainPath + path);

        return dir.isDirectory();
    }

}
