package upv.pauchorroyanguas.com.encryptorpcy.Utils;

import android.media.MediaPlayer;
import android.media.MediaRecorder;
import android.os.Environment;
import android.util.Log;

import java.io.File;
import java.io.IOException;

/**
 * Author: Pau Chorro
 * Date: 01/11/15
 *
 * VoiceNoteUtil TODO Impl
 */
public class VoiceNoteUtil {

    //Const
    private static final String LOG_TAG = "AudioRecordTest";
    private static String mainPath = Environment.getExternalStorageDirectory() +
            "/";

    //Fields
    private static String mFileName = null;

    //private Button mRecordButton = null;
    private MediaRecorder mRecorder = null;

    //private Button   mPlayButton = null;
    private MediaPlayer mPlayer = null;

    private boolean mStartRecording;
    private boolean mStartPlaying;

    /**
     * Constructor of the class,  it initialize mFileName to the externalStorageDirectory path
     * concat with the name of the file
     * @param nameFile String Name of the file
     */
    public VoiceNoteUtil(String nameFile) {
        //Old
        //mFileName = Environment.getExternalStorageDirectory().getAbsolutePath();
        //New

        File storageDir = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_MUSIC);

        if(!storageDir.exists())
            storageDir.mkdir();

        //mFileName = mainPath;
        mFileName = storageDir.getAbsolutePath();
        Log.e("ososo" , mFileName);
        mFileName +=  nameFile + ".3gp";
        Log.e("ososo" , mFileName);
        mStartRecording = true;
        mStartPlaying = true;
    }

    /**
     * Get the FileName of the Voice Note
     * @return String the File name of the VoiceNote
     */
    public static String getmFileName() {

        return mFileName;
    }

    /**
     * Set the File Name of the voice note
     * @param mFileName String New file name to set the old
     */
    public static void setmFileName(String mFileName) {
        VoiceNoteUtil.mFileName = mFileName;
    }

    /**
     * Start the record if the start is true, if it is not, it stop the record
     * @param start Boolean to start o stop de record
     */
    public void onRecord(boolean start) {
        if (start) {
            startRecording();
        } else {
            stopRecording();
        }
    }

    /**
     * Start the voice note if the boolean start is true, if it is false, stop the playing
     * @param start Boolean to start o stop the playing record
     */
    public void onPlay(boolean start) {
        if (start) {
            startPlaying();
        } else {
            stopPlaying();
        }
    }

    /**
     * It's intialize a mediaplayer with the file and it will prepare and start.
     */
    private void startPlaying() {
        mPlayer = new MediaPlayer();
        try {


            mPlayer.setDataSource(mFileName);
            mPlayer.prepare();
            mPlayer.start();
            Log.e(LOG_TAG, "startPlaying() true");
        } catch (IOException e) {
            Log.e(LOG_TAG, "startPlaying() failed");
        }
    }

    /**
     * It will stop the mediaplayer
     * Then It is released after stop and after it is put to null
     */
    public void stopPlaying() {
        mPlayer.release();
        Log.e(LOG_TAG, "stopPlaying() true");
        mPlayer = null;
    }

    /**
     * It will start the recording, creating a MediaRecorder
     * First, it will put the audio source from Micro
     * Second, the OutputFormat it will be THREE_HPP
     * Third, the output file will be the main file of the class mFilename
     * Fourth, it set the audio enocder to AMR_NB
     * Finally, it will prepare the MediaRecorder, and if there will not be an error
     * it will be start
     */
    private void startRecording() {

        mRecorder = new MediaRecorder();
        mRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);
        mRecorder.setOutputFormat(MediaRecorder.OutputFormat.THREE_GPP);
        mRecorder.setOutputFile(mFileName);
        mRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AMR_NB);

        try {
            mRecorder.prepare();

        } catch (IOException e) {
            Log.e(LOG_TAG, "startRecording() failed");
        }

        mRecorder.start();
        Log.e(LOG_TAG, "startRecording() true");
    }

    /**
     * Stop the media recorder, and put to null to call the bin collector
     * Then It is released after stop and after it is put to null
     */
    private void stopRecording() {
        mRecorder.stop();
        Log.e(LOG_TAG, "stopRecording() true");
        mRecorder.release();
        mRecorder = null;
    }

    /**
     * Boolean to know if is recording or not
     * @return boolean The field mStartRecording
     */
    public boolean ismStartRecording() {
        return mStartRecording;
    }

    /**
     * Set the boolean mStartRecording to true if it is false, or false if it is true
     * @param mStartRecording (boolean) param to set the field
     */
    public void setmStartRecording(boolean mStartRecording) {
        this.mStartRecording = mStartRecording;
    }

    /**
     *  Boolean to know if is playing or not
     * @return boolean The field mStartPlaying
     */
    public boolean ismStartPlaying() {
        return mStartPlaying;
    }

    /**
     * Set the boolean mStartPlaying to true if it is false, or false if it is true
     * @param mStartPlaying (boolean) param to set the field
     */
    public void setmStartPlaying(boolean mStartPlaying) {
        this.mStartPlaying = mStartPlaying;
    }
}
